#! /usr/bin/perl
#
#  Purpose:
#  FlowTracker_Management.cgi creates the web page for managaing
#  Flowtrackings (i.e., Modify, Remove, Archive, Restart.)
#
#  Description:
#
#  FlowTracker_Management.cgi parses through existing FlowTracker 
#  Filter files and creates the management web-page.
#
#  Input arguments:
#  Name                 Description
#  -----------------------------------------------------------------------
#  none      
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  05/08/2012      4.0     Major upgrade for IPFIX/v9 using SiLK,
#                                       New User Interface (reuses some FT_Main)
#  J. Loiacono  04/15/2013      4.1     Fixed rename/remove to include Dashboard
#  J. Loiacono  09/11/2013      4.2.1   Minor formatting changes
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;
use FlowViewer_UI;

if ($debug_tracker eq "Y") { open (DEBUG,">>$work_directory/DEBUG_TRACKER_M"); }
if ($debug_tracker eq "Y") { print DEBUG "In FlowTracker_Management.cgi\n"; }

# Retrieve parameters for a 'Remove' or 'Revise' action

($action,$tracking_label,$device_revision) = split(/\^/,$ENV{'QUERY_STRING'});
if ($action =~ "%5E") { ($action,$tracking_label) = split("%5E",$action); }
if ($debug_tracker eq "Y") { print DEBUG "action: $action  tracking_label: $tracking_label  device_revision: $device_revision\n"; }
if ($action eq "List")    { $action = "List Trackings"; }
if ($action eq "Solicit") { $action = "Solicit Revision"; }
if ($action eq "Revise")  { $action = "Revise Tracking"; }
if ($action eq "SolicitN"){ $action = "Solicit Rename"; }
if ($action eq "Rename")  { $action = "Rename Tracking"; }
if ($action eq "Archive") { $action = "Archive Tracking"; }
if ($action eq "Restart") { $action = "Restart Tracking"; }
if ($action eq "Remove")  { $action = "Remove Tracking"; }
if ($action eq "Proceed") { $action = "Proceed with Removal"; }
if ($device_revision ne "") { ($device_revision,$device_name_revision) = split(/=/,$device_revision); } chop($device_name_revision);

$tracking_label =~ s/~/ /g;
if ($debug_tracker eq "Y") { print DEBUG "action: $action   tracking_label: $tracking_label\n"; }

$tm_tracking_file = $tracking_label;
$tm_tracking_file =~ s/^\s+//;
$tm_tracking_file =~ s/\s+$//;
$tm_tracking_file =~ s/\&/-/g;
$tm_tracking_file =~ s/\//-/g;
$tm_tracking_file =~ s/\(/-/g;
$tm_tracking_file =~ s/\)/-/g;
$tm_tracking_file =~ s/\./-/g;
$tm_tracking_file =~ s/\s+/_/g;
$tm_tracking_file =~ tr/[A-Z]/[a-z]/;

$filter_file    = $filter_directory  ."/". $tm_tracking_file .".fil";
$group_file     = $filter_directory  ."/". $tm_tracking_file .".grp";
$archive_file   = $filter_directory  ."/". $tm_tracking_file .".archive";
$rrdtool_file   = $rrdtool_directory ."/". $tm_tracking_file .".rrd";

$html_directory = $tracker_directory ."/". $tm_tracking_file;
$html_file      = $html_directory ."/index.html";

&create_UI_top("FlowTracker");
&create_UI_service("FlowTracker","service_top",$filter_hash);

# Create the FlowTracking Management Content

if ($action eq "List Trackings") {

	&create_UI_sides("FlowTracker");
	print " <div id=content_scroll>\n";
	print "<br><b>Individual Trackings</b><br><br>\n\n";
	
	@filter_list = <$filter_directory/*>;
	@filter_list = sort(@filter_list);
	
	print "  <table>\n";
	
	foreach $filter_file (@filter_list) {
		
	        $tm_tracking_file = $filter_file; 
	        $tm_tracking_file =~ s#.*/##; 
	
	        open (FILTERS,"<$filter_file"); 
	        while (<FILTERS>) { 
	                chop;    
	                $key = substr($_,0,8);
	                if ($key eq " input: ") {
	
	                        ($input,$field,$field_value) = split(/: /);
	
	                        if ($field eq "tracking_label")   { 
					$filter_label = $field_value;
	
					$filter_file   = "$filter_directory/$tm_tracking_file"; 
					($tracking_prefix,$tracking_suffix) = split(/\./,$tm_tracking_file);
					$filter_hash = "TR_" . $tracking_prefix;
	
					$file_label = $filter_label;
					$file_label =~ s/ /~/g;
	
					$tracking_link = "<a href=$cgi_bin_short/FlowTracker_Display.cgi?filter_hash=$filter_hash>$filter_label</a>";
					$revise_link   = "<a href=$cgi_bin_short/FlowTracker_Management.cgi?Solicit^$file_label>Revise</a>";
					$rename_link   = "<a href=$cgi_bin_short/FlowTracker_Management.cgi?SolicitN^$file_label>Rename</a>";
					$remove_link   = "<a href=$cgi_bin_short/FlowTracker_Management.cgi?Remove^$file_label>Remove</a>";
	
					if ($tracking_suffix eq "archive") {
						$num_archive++;
						$stop_link = "<a href=$cgi_bin_short/FlowTracker_Management.cgi?Restart^$file_label>Restart</a>";
						$rename_link = "<a href=$cgi_bin_short/FlowTracker_Management.cgi?SolicitN^$file_label>Rename</a>";
						$revise_link = "";
						$archive_link = $tracking_link ."^^". $revise_link ."^^". $rename_link ."^^". $stop_link ."^^". $remove_link;
						$archive_links{$num_archive} = $archive_link;
						next;
					} elsif ($tracking_suffix eq "grp") {
						$num_group++;
						$revise_link = "<a href=$cgi_bin_short/FlowTracker_Group.cgi?Solicit^$file_label>Revise</a>";
						$rename_link = "<a href=$cgi_bin_short/FlowTracker_Management.cgi?SolicitN^$file_label>Rename</a>";
						$stop_link = "<a href=$cgi_bin_short/FlowTracker_Management.cgi?Archive^$file_label>Archive</a>"; 
						$group_link = $tracking_link ."^^". $revise_link ."^^". $rename_link ."^^". $stop_link ."^^". $remove_link;
						$group_links{$num_group} = $group_link;
						next;
					} else {
						$stop_link = "<a href=$cgi_bin_short/FlowTracker_Management.cgi?Archive^$file_label>Archive</a>"; 
					}
	
					print "  <tr>\n";
					print "  <td class=tr_label>$tracking_link</td>\n";
					print "  <td class=tr_action>$revise_link</td>\n";
					print "  <td class=tr_action>$rename_link</td>\n";
					print "  <td class=tr_action>$stop_link</td>\n";
					print "  <td class=tr_action>$remove_link</td>\n";
					print "  </tr>\n";
	
				} else {
					next;
				}
			}
		}
	} 
	
	print "</table>";
	print "<table>";
	
	if ($num_group > 0) {
		print "   <br><b>Group Trackings</b><br><br>\n\n";
		@sorted_groups = sort { $a <=> $b} (keys %group_links);
		foreach $link_index (@sorted_groups) {
			($tracking_link,$revise_link,$rename_link,$stop_link,$remove_link) = split(/\^\^/,$group_links{$link_index});
			print "  <tr>\n";
			print "  <td class=tr_label>$tracking_link</td>\n";
			print "  <td class=tr_action>$revise_link</td>\n";
			print "  <td class=tr_action>$rename_link</td>\n";
			print "  <td class=tr_action>$stop_link</td>\n";
			print "  <td class=tr_action>$remove_link</td>\n";
			print "  </tr>\n";
		}
	}
	
	print "</table>";
	print "<table>";
	
	if ($num_archive > 0) {
		print "   <br><b>Archived Trackings</b><br><br>\n\n";
		@sorted_archives = sort { $a <=> $b } (keys %archive_links);
		foreach $link_index (@sorted_archives) {
			($tracking_link,$revise_link,$rename_link,$stop_link,$remove_link) = split(/\^\^/,$archive_links{$link_index});
			print "  <tr>\n";
			print "  <td class=tr_label>$tracking_link</td>\n";
			print "  <td class=tr_action>$revise_link</td>\n";
			print "  <td class=tr_action>$rename_link</td>\n";
			print "  <td class=tr_action>$stop_link</td>\n";
			print "  <td class=tr_action>$remove_link</td>\n";
			print "  </tr>\n";
		}
	}
	
	print "</table>";
	print " </div>\n";
}

elsif ($action eq "Solicit Revision") {

        $filter_hash = $tm_tracking_file;
	$filter_hash = "TR_" . $filter_hash;
	if ($device_revision ne "") { $filter_hash .= "^$device_revision"; }

        # Retrieve form input for Tracking to be modified

	&create_UI_sides("FlowTracker");
	print " <div id=content>\n";
	&create_filtering_form("FlowTracker_Sol",$filter_hash);
	&create_tracking_form($filter_hash);
	&create_SiLK_form($filter_hash);
	&create_revision_form($filter_hash);
	&create_submit_buttons("FlowTracker_Sol");

	print " </div>\n";
}

elsif ($action eq "Solicit Rename") {

	&create_UI_sides("FlowTracker");
	print " <div id=content>\n";

        $button_text = "Rename Tracking";
        print "  <center>\n";
	print "  <br><br>Current FlowTracking Name:<font color=$filename_color><b><i> $tracking_label </i></b></font>\n";
        print "  <form action=$cgi_bin_short/FlowTracker_Management.cgi?Rename^$tm_tracking_file method=POST>\n";
        print "  <br><br><br>\n";
        print "  New FlowTracking Name (Maximum 80 characters):\n";
        print "  <br><br>\n";
        print "  <span class=input_center>\n";
        print "  <input type=text style=\"clear: both\; width: 30em;\" name=new_name value=\"$new_name\">\n";
        print "  </span>\n";
        print "  <br><br><br>\n";
        print "  <input class=buttonc type=submit value=\"Rename the FlowTracking\">\n";
        print "  <br><br><br>\n";
        print "  </form>\n";
        print "  </table>";

        print "  </center>\n";
        print " </div>\n";
}

elsif ($action eq "Rename Tracking") {

	# Retrieve new FlowTracking name

        read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
        @pairs = split(/&/, $buffer);
        foreach $pair (@pairs) {
            ($name, $value) = split(/=/, $pair);
            $value =~ tr/+/ /;
            $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
            $FORM{$name} = $value;
        }

        $new_tracking_label = $FORM{new_name};
        $new_tracking_label = substr($new_tracking_label,0,80);

	# Retrieve old FlowTracking name

        if (-e $group_file) { 
		$old_tracking_file = $filter_directory ."/". $tm_tracking_file .".grp";
	} elsif (-e $archive_file) {
		$old_tracking_file = $filter_directory ."/". $tm_tracking_file .".archive";
		$old_rrdtool_file = $rrdtool_directory ."/". $tm_tracking_file .".archive";
	} else {
		$old_tracking_file = $filter_directory ."/". $tm_tracking_file .".fil";
		$old_rrdtool_file = $rrdtool_directory ."/". $tm_tracking_file .".rrd";
	}

	$old_html_directory = "$tracker_directory/$tm_tracking_file";

        open  (OLD_FILTER,"<$old_tracking_file");
        while (<OLD_FILTER>) {
                chop;
                $key = substr($_,0,8);
                if ($key eq " input: ") {
                        ($input,$field,$field_value) = split(/: /);
                        if ($field eq "tracking_label") { $old_tracking_label = $field_value; }
                        if ($field eq "tracking_type")  { $old_tracking_type  = $field_value; }
                }
        }
        close (OLD_FILTER);
	$is_group = 0; if ($old_tracking_type eq "Group") { $is_group = 1; }

	# Define new Filter, RRDtool, and HTML files and directory

	$tn_tracking_file = $new_tracking_label;
	$tn_tracking_file =~ s/^\s+//;
	$tn_tracking_file =~ s/\s+$//;
	$tn_tracking_file =~ s/\&/-/g;
	$tn_tracking_file =~ s/\//-/g;
	$tn_tracking_file =~ s/\(/-/g;
	$tn_tracking_file =~ s/\)/-/g;
	$tn_tracking_file =~ s/\./-/g;
	$tn_tracking_file =~ s/\s+/_/g;
	$tn_tracking_file =~ tr/[A-Z]/[a-z]/;
	
        if (-e $group_file) { 
		$new_tracking_file = $filter_directory ."/". $tn_tracking_file .".grp";
		$new_rrdtool_file = "";
	} elsif (-e $archive_file) {
		$new_tracking_file = $filter_directory ."/". $tn_tracking_file .".archive";
		$new_rrdtool_file = $rrdtool_directory ."/". $tn_tracking_file .".archive";
	} else {
		$new_tracking_file = $filter_directory ."/". $tn_tracking_file .".fil";
		$new_rrdtool_file = $rrdtool_directory ."/". $tn_tracking_file .".rrd";
	}

	$new_tracking_filter  = $filter_directory  ."/". $tn_tracking_file .".fil";
	$new_tracking_group   = $filter_directory  ."/". $tn_tracking_file .".grp";
	$new_tracking_archive = $filter_directory  ."/". $tn_tracking_file .".archive";

	$name_in_use = 0;
	if (-e $new_tracking_filter)  { $name_in_use = 1; $found_file = $new_tracking_filter; }
	if (-e $new_tracking_group)   { $name_in_use = 1; $found_file = $new_tracking_group; }
	if (-e $new_tracking_archive) { $name_in_use = 1; $found_file = $new_tracking_archive; }

	# Check to see if name available

	if (($name_in_use) || ($new_tracking_label eq ""))  {

		&create_UI_sides("FlowTracker");
		print " <div id=content>\n";
	
	        $button_text = "Rename Tracking";
	        print "  <center>\n";
		print "  <br><br>Current FlowTracking Name:<font color=$filename_color><b><i> $old_tracking_label </i></b></font>\n";
	        print "  <form action=$cgi_bin_short/FlowTracker_Management.cgi?Rename^$tm_tracking_file method=POST>\n";
	        print "  <br><br>\n";
		if ($name_in_use) {

                	$found_filter = $found_file;
                	$found_filter =~ s#.*/##;

        		print " <table>";
			print " <tr><td align=right><b>Error:&nbsp&nbsp</b></td><td align=left>The new Tracking name is in use, please provide a different name.</td></tr>\n";
			print " <tr><td align=right><b>Existing Filter:&nbsp&nbsp</b></td><td align=left>$found_filter</td></tr>\n";
			print " <tr><td>&nbsp</td></tr>\n";
			print " <tr><td>&nbsp</td></tr>\n";
        		print " </table>";
		} elsif ($new_tracking_label eq "") {
        		print " <table>";
			print " <tr><td align=right><b>Error:&nbsp&nbsp</b></td><td align=left>No name supplied.</td></tr>\n";
			print " <tr><td>&nbsp</td></tr>\n";
			print " <tr><td>&nbsp</td></tr>\n";
        		print " </table>";
		}
	        print "  New FlowTracking Name (Maximum 80 characters):\n";
	        print "  <br><br>\n";
	        print "  <span class=input_center>\n";
	        print "  <input type=text style=\"clear: both\; width: 30em;\" name=new_name value=\"$new_name\">\n";
	        print "  </span>\n";
	        print "  <br><br><br>\n";
	        print "  <input class=buttonc type=submit value=\"Rename the FlowTracking\">\n";
	        print "  <br><br><br>\n";
	        print "  </form>\n";
	
	        print "  </center>\n";
	        print " </div>\n";

		&create_UI_service("FlowTracker","service_bottom",$filter_hash);
		&finish_the_page("FlowTracker");

		exit;
	}

	$new_html_directory = "$tracker_directory/$tn_tracking_file";

	# Change Tracking label in new filter file

        open (OLD_FILTER,"<$old_tracking_file");
        open (NEW_FILTER,">$new_tracking_file");
        while (<OLD_FILTER>) {
                $key = substr($_,0,8);
                if ($key eq " input: ") {
                        ($input,$field,$field_value) = split(/: /);
                        if ($field eq "tracking_label") {
				print NEW_FILTER " input: tracking_label: $new_tracking_label\n";
                        } else {
				print NEW_FILTER $_;
                        }
                } else {
			print NEW_FILTER $_;
		}
        }
        close (OLD_FILTER);
        close (NEW_FILTER);
	chmod $filter_file_perms, $new_tracking_file;

	# Remove old filter file, rename RRDtool file and FlowTracking HTML directory

	$remove_command = "rm \-f $old_tracking_file";
	system($remove_command);

	if ($new_rrdtool_file ne "") {
		$move_command = "mv $old_rrdtool_file $new_rrdtool_file";
		system($move_command);
		chmod $rrd_file_perms, $new_rrdtool_file;
	}

	$move_command = "mv $old_html_directory $new_html_directory";
	system($move_command);
	chmod $html_dir_perms, $new_html_directory;

	# Rename Dashboard file if it exists
	
	$db_tracking_file = $old_tracking_label;
	$db_tracking_file =~ s/^\s+//;
	$db_tracking_file =~ s/\s+$//;
	$db_tracking_file =~ s/\&/-/g;
	$db_tracking_file =~ s/\//-/g;
	$db_tracking_file =~ s/\(/-/g;
	$db_tracking_file =~ s/\)/-/g;
	$db_tracking_file =~ s/\./-/g;
	$db_tracking_file =~ s/\s+/_/g;
	$db_tracking_file =~ tr/[A-Z]/[a-z]/;
	
	$len_old_label = length($db_tracking_file);
	
        @dashboard_files = <$dashboard_directory/*>;
	foreach $dashboard_file (@dashboard_files) {
		$file_w_suffix = $dashboard_file;
		$file_w_suffix =~ s#.*/##;
		$last_underscore = rindex($file_w_suffix,"_");
		$remove_underscore = substr($file_w_suffix,0,$last_underscore);
		$next_last_underscore = rindex($remove_underscore,"_");
		$dashboard_label  = substr($file_w_suffix,0,$next_last_underscore);
		$dashboard_suffix = substr($file_w_suffix,$next_last_underscore,255);
		if ($dashboard_label eq $db_tracking_file) {
			$new_dashboard_file = $dashboard_directory . "/" . $tn_tracking_file . $dashboard_suffix;
			$move_command = "mv $dashboard_file $new_dashboard_file";
			system($move_command);
			chmod $html_dir_perms, $new_dashboard_file;
		}
	}
	
	# Rename this Individual Tracking in any groups of which it is a component

	@containing_groups = ();
	@containing_group_names = ();

        @group_files = <$filter_directory/*.grp>;
        if ((!$is_group) && (@group_files != ())) {

                foreach $group_file (@group_files) {
		        open (GROUP_FILE,"<$group_file");
		        while (<GROUP_FILE>) {
	                	$key = substr($_,0,8);
		                if ($key eq " input: ") { 
                                        if (/tracking_label/) { 
						($record_type,$input_type,$group_label) = split(/: /,$_);
						if ($containing_group) {
							push(@containing_group_names,$group_label);
							$containing_group = 0;
						}
					}
					next;
		                } else {
		                        ($component_position,$component_label,$component_color) = split(/\^/);
		                        if ($old_tracking_label eq $component_label) { 
						push(@containing_groups,$group_file);
						$containing_group = 1;
					}
				}
			}
			close(GROUP_FILE);
		}

		$temp_group = "$work_directory/temp_group.grp";
                foreach $group_file (@containing_groups) { 

		        open (OLD_GROUP,"<$group_file");
		        open (TEMP_GROUP,">$temp_group");
		        while (<OLD_GROUP>) {
	                	$key = substr($_,0,8);
		                if ($key eq " input: ") { 
					print TEMP_GROUP $_;
		                } else {
		                        ($component_position,$component_label,$component_color) = split(/\^/);
		                        if ($old_tracking_label eq $component_label) { 
						$group_member = $component_position ."^". $new_tracking_label ."^". $component_color;
						print TEMP_GROUP "$group_member";
					} else {
						print TEMP_GROUP $_;
					}
				}
			}
			close(OLD_GROUP);
			close(TEMP_GROUP);

			$move_command = "mv $temp_group $group_file";
        		system($move_command);
			chmod $filter_file_perms, $group_file;
		}
	}

	# Output the changed information

	&create_UI_sides("FlowTracker");
	print " <div id=content>\n";
	print "  <br><br>\n";
        print "  <table>";
	print "  <tr><td align=right>&nbsp&nbsp</td><td align=left><b>FlowTracking has been Successfully Renamed</b></td></tr>\n";
	print "  <tr><td>&nbsp</td></tr>\n";
	print "  <tr><td align=right>Old Tracking File: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$old_tracking_file</i></b></font></td></tr>\n";
        if (!$is_group) { 
	print "  <tr><td align=right>Old RRDtool File: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$old_rrdtool_file</i></b></font></td></tr>\n";
	}
	print "  <tr><td align=right>Old HTML Directory: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$old_html_directory</i></b></font></td></tr>\n";
	print "  <tr><td align=right>Old Tracking Name: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$old_tracking_label</i></b></font></td></tr>\n";
	print "  <tr><td>&nbsp</td></tr>\n";
	print "  <tr><td align=right>New Tracking File: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$new_tracking_file</i></b></font></td></tr>\n";
        if (!$is_group) { 
	print "  <tr><td align=right>New RRDtool File: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$new_rrdtool_file</i></b></font></td></tr>\n";
	}
	print "  <tr><td align=right>New HTML Directory: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$new_html_directory</i></b></font></td></tr>\n";
	print "  <tr><td align=right>New Tracking Name: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$new_tracking_label</i></b></td></tr>\n";

	if (@containing_group_names != ()) {
		print "  <tr><td>&nbsp</td></tr>\n";
		print "  <tr><td align=right>&nbsp&nbsp</td><td align=left>The FlowTracking reference was renamed in the following Groups</td></tr>\n";
		print "  <tr><td>&nbsp</td></tr>\n";
	}
	foreach $containing_group_name (@containing_group_names) {
		print "  <tr><td align=right>Updated Group: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$containing_group_name</i></b></font></td></tr>\n";
	}
		
        print "  </table>\n";
	print "  <br>\n";
        print "  <table>\n";
       	print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td><form method=post action=\"$cgi_bin_short/FlowTracker_Management.cgi?List\">\n";
        print "  <button class=links type=submit>Return</button></form></td></tr>\n";
        print "  </table>\n";
        print "  </div>\n";
}

elsif ($action eq "Remove Tracking") {

        if (-e $group_file) {
                $is_group = 1;
        	$filter_group = $filter_file;
        	$filter_group =~ s/\.fil/\.grp/;
	} elsif (-e $archive_file) {
		$is_archive = 1;
	        open  (FILTER_ARCHIVE,"<$archive_file");
	        while (<FILTER_ARCHIVE>) {
	                chop;
	                $key = substr($_,0,8);
	                if ($key eq " input: ") {
	                        ($input,$field,$field_value) = split(/: /);
	                        if ($field eq "tracking_type")  { $archive_type  = $field_value; }
	                }
	        }
	        close (FILTER_ARCHIVE);
		$is_group = 0; if ($archive_type eq "Group") { $is_group = 1; }
        } else {
                @group_files = <$filter_directory/*.grp>;
                foreach $group_file (@group_files) {
                        $is_a_member = 0;
                        open (GROUP_FILE,"<$group_file");
                        while (<GROUP_FILE>) {
                                $key = substr($_,0,8);
                                if ($key eq " input: ") {
                                        if (/tracking_label/) { ($record_type,$input_type,$group_label) = split(/: /,$_); }
                                        next;
                                } else {
                                        ($component_position,$component_label,$component_color) = split(/\^/);
                                        if ($tracking_label eq $component_label) { $is_a_member = 1; }
                                }
                        }
                        if ($is_a_member) { push(@member_groups,$group_label); }
                }
        }

        # Output the summary web page

        $filter_file  =~ s#.*/##;
        $rrdtool_file =~ s#.*/##;

        $html_file = $html_directory;
        $html_file =~ s#.*/##;

        $file_label = $tracking_label;
        $file_label =~ s/ /~/g;

	&create_UI_sides("FlowTracker");
	print " <div id=content_wide>\n";
	print "<br><b>Remove FlowTracking</b><br><br>\n\n";

        print "  <br>\n";
        print "  <table>\n";
        print "  <tr><td colspan=2>If you proceed, all files and directories associated with:</td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td width=140 align=right><b>FlowTracking: &nbsp&nbsp</b></td><td align=left><font color=$filename_color><b><i>$tracking_label</i></b></font></td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td colspan=2>will be moved to your Work directory, under subdirectory:</td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td width=140 align=right><b>Subdirectory: &nbsp&nbsp</b></td><td align=left><font color=$filename_color><b><i>$work_directory</i></b></font></td></tr>\n";
        print "  <tr><td align=right> &nbsp&nbsp</td><td><font color=$filename_color><b><i>/$html_file</i></b></td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td colspan=2>If you proceed in error, you will still be able recreate this FlowTracking by moving the FlowTracking files back to the</td></tr>\n";
        print "  <tr><td colspan=2>operational directories, verifying that directory permissions are correct, and recreating and repopulating (.png files)</td></tr>\n";
        print "  <tr><td colspan=2>the Tracker HTML sub-directory. That would be:</td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";

	$temp_filter = $filter_file;
       	if ($is_group)   { 
		$temp_filter =~ s/\.fil/\.grp/;
       		if ($is_archive) { $temp_filter =~ s/\.grp/\.archive/; }
	}
       	if ($is_archive) { $temp_filter =~ s/\.fil/\.archive/; }

        if ($is_group) {
        	print "  <tr><td align=right>1) Move: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$work_directory/$temp_filter</i></b></td></tr>\n";
        	print "  <tr><td align=right>Back to: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$filter_directory</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>2) Recreate & &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$tracker_directory/$html_file</i></b></font> directory.</td></tr>\n";
        	print "  <tr><td align=right>Repopulate: &nbsp&nbsp</td><td align=left>&nbsp</td></tr>\n";
        } else {
		print "  <tr><td align=right>1) Move: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$work_directory/$html_file/$temp_filter</i></b></td></tr>\n";
        	print "  <tr><td align=right>Back to: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$filter_directory</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>2) Move: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$work_directory/$html_file/$rrdtool_file</i></b></font></td></tr>\n";
        	print "  <tr><td align=right>Back to: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$rrdtool_directory</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>3) &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp</td><td>If this Tracking was on the Dashboard, either re-create the Dashnoard entry, or copy Dashboard file</i></b></font></td></tr>\n";
        	print "  <tr><td align=right>Back to: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$dashboard_directory</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>4) Recreate & &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$tracker_directory/$html_file</i></b></font> &nbsp directory.</td></tr>\n";
        	print "  <tr><td align=right>Repopulate: &nbsp&nbsp</td><td align=left>&nbsp</td></tr>\n";
        }
        print "  <tr><td>&nbsp</td></tr>\n";

        if ($#member_groups > -1) {
        	print "  <tr><td colspan=2>This tracking was found to be a component of the following Groups:</td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
                foreach $member_group (@member_groups) { print "  <tr><td>&nbsp</td><td align=left><font color=$filename_color><b><i>$member_group</i></b></font></td></tr>\n"; }
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td colspan=2>This tracking will be removed from each of the Groups listed above.</td></tr>\n";
        }
        print "  </table>\n";

        print "  <table>\n";
       	print "  <tr><td>&nbsp</td></tr>\n";
       	print "  <tr><td>&nbsp</td></tr>\n";
	print "  <tr>";
        print "  <td align=left><form method=post action=\"$cgi_bin_short/FlowTracker_Management.cgi?Proceed^$file_label\">\n";
        print "  <button class=links type=submit>Proceed</button></form></td>\n";
       	print "  <td>&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp</td>\n";
        print "  <td align=center><form method=post action=\"$cgi_bin_short/FlowTracker_Management.cgi?List\">\n";
        print "  <button class=links type=submit>Return</button></form></td>\n";
	print "  </tr>";

        print "  </table>\n";
        print "  </div>\n";
}

elsif ($action eq "Proceed with Removal") {

        if (!-e $work_directory) {
                mkdir $work_directory, $work_dir_perms || die "Cannot mkdir Work directory: $work_directory: $!";
                chmod $work_dir_perms, $work_directory;
        }

        if (-e $group_file) { $is_group = 1; }

	if (-e $archive_file) {
		$is_archive = 1;
	        open  (FILTER_ARCHIVE,"<$archive_file");
	        while (<FILTER_ARCHIVE>) {
	                chop;
	                $key = substr($_,0,8);
	                if ($key eq " input: ") {
	                        ($input,$field,$field_value) = split(/: /);
	                        if ($field eq "tracking_type")  { $archive_type  = $field_value; }
	                }
	        }
	        close (FILTER_ARCHIVE);
		$is_group = 0; if ($archive_type eq "Group") { $is_group = 1; }
        }

	# Create a temporary directory in $work_directory

        $html_file = $html_directory;
        $html_file =~ s#.*/##;
	$temp_tracker_dir = "$work_directory/$html_file";
	$mkdir_command = "mkdir $temp_tracker_dir";
        system($mkdir_command);
        chmod $work_dir_perms, $temp_tracker_dir;

        # Remove directory and contents

	$copy_command = "cp $html_directory/\* $temp_tracker_dir";
        system($copy_command);
        $rmdir_command = "rm -rf $html_directory";
        system($rmdir_command);

        # Move filter (or Archived) file to temp directory

        $filter_archive = $filter_file;
        $filter_archive =~ s/\.fil/\.archive/;

        $filter_group = $filter_file;
        $filter_group =~ s/\.fil/\.grp/;

        $mv_command = "mv  $filter_file $temp_tracker_dir";
        system($mv_command);

        $mv_command = "mv  $filter_group $temp_tracker_dir";
        system($mv_command);

        $mv_command = "mv  $filter_archive $temp_tracker_dir";
        system($mv_command);

        # Move rrd (or Archived) file to work directory

        $rrdtool_archive = $rrdtool_file;
        $rrdtool_archive =~ s/\.rrd/\.archive/;

        $mv_command = "mv  $rrdtool_archive $rrdtool_file";
        system($mv_command);

        $mv_command = "mv  $rrdtool_file $temp_tracker_dir";
        system($mv_command);

	# Remove Dashboard file if it is there

        @dashboard_files = <$dashboard_directory/*>;
	foreach $dashboard_file (@dashboard_files) {
		$file_w_suffix = $dashboard_file;
		$file_w_suffix =~ s#.*/##;
		$last_underscore = rindex($file_w_suffix,"_");
		$position = substr($file_w_suffix,$last_underscore+1,1);
		$remove_underscore = substr($file_w_suffix,0,$last_underscore);
		$next_last_underscore = rindex($remove_underscore,"_");
		$dashboard_label  = substr($file_w_suffix,0,$next_last_underscore);
		$dashboard_suffix = substr($file_w_suffix,$next_last_underscore,255);
		if ($dashboard_label eq $tm_tracking_file) {
			open(VARIABLES,">$work_directory/FT_Dash_remove");
			$dash_action = "dash_action=RemoveI_" . "$position";
			print VARIABLES "$dash_action";
			close (VARIABLES);
			$ENV{QUERY_STRING}="$dash_action";
			$ENV{CONTENT_LENGTH}=21;
			$command = "$cgi_bin_directory/FlowTracker_Dashboard.cgi < $work_directory/FT_Dash_remove";
			system($command);
			$rm_command = "rm -f $work_directory/FT_Dash_remove";
			if ($debug_files ne "Y") { system($rm_command); }
		}
	}

        # Remove this Tracking from any groups of which it is a component

        if (!$is_group) {

                @group_files = <$filter_directory/*.grp>;

                foreach $group_file (@group_files) {

                        @group_lines = ();
                        $found_removal = 0;

                        open(GROUP_FILE,"<$group_file");
                        while (<GROUP_FILE>) {

                                $key = substr($_,0,8);
                                push(@group_lines,$_);

                                if ($key eq " input: ") {
                                        if (/tracking_label/) { ($record_type,$input_type,$group_label) = split(/: /,$_); }
                                        next;
                                } else {
                                        ($component_position,$component_label,$component_color) = split(/\^/);
                                        $component_hundreds = substr($component_position,0,1);

                                        if ($tracking_label eq $component_label) {
                                                ($removed_position,$removed_label,$removed_color) = split(/\^/);
                                                $removed_hundreds = substr($removed_position,0,1);
                                                $found_removal = 1;
                                                pop(@group_lines);
                                                next;
                                        }

                                        if (($found_removal) && ($component_hundreds eq $removed_hundreds)) {
                                                pop(@group_lines);
                                                $component_position--;
                                                $replaced_line = $component_position ."^". $component_label ."^". $component_color;
                                                push(@group_lines,$replaced_line);
                                        }
                                }
                        }
                        close (GROUP_FILE);

                        if ($found_removal) {
                                push(@removed_groups,$group_label);
                                open(GROUP_FILE,">$group_file");
                                foreach $group_line (@group_lines) { print GROUP_FILE $group_line; }
                                close (GROUP_FILE);
                        }
                }
        }

        # Output the summary web page

        $filter_file  =~ s#.*/##;
        $rrdtool_file =~ s#.*/##;

	&create_UI_sides("FlowTracker");
	print " <div id=content_wide>\n";
	print "<br><b>Proceeding with FlowTracking Removal</b><br><br>\n\n";

        print "  <br>\n";
        print "  <table>\n";
        print "  <tr><td colspan=2>All files and directories associated with:</td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td width=140 align=right><b>FlowTracking: &nbsp&nbsp</b></td><td align=left><font color=$filename_color><b><i>$tracking_label</i></b></font></td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td colspan=2>have been moved to your Work directory, under subdirectory:</td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td width=140 align=right><b>Subdirectory: &nbsp&nbsp</b></td><td align=left><font color=$filename_color><b><i>$work_directory</i></b></font></td></tr>\n";
        print "  <tr><td align=right> &nbsp&nbsp</td><td><font color=$filename_color><b><i>/$html_file</i></b></td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td colspan=2>If necessary, you can recreate this FlowTracking by moving the FlowTracking files back to the operational directories,</td></tr>\n";
        print "  <tr><td colspan=2>recreate and repopulate (.png files) the Tracking sub-directory and verify that directory permissions are correct.</td></tr>\n";
        print "  <tr><td colspan=2>That would be:</td></tr>\n";
        print "  <tr><td>&nbsp</td></tr>\n";

	$temp_filter = $filter_file;
       	if ($is_group)   { 
		$temp_filter =~ s/\.fil/\.grp/;
       		if ($is_archive) { $temp_filter =~ s/\.grp/\.archive/; }
	}
       	if ($is_archive) { $temp_filter =~ s/\.fil/\.archive/; }

        if ($is_group) {
        	print "  <tr><td align=right>1) Move: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$work_directory/$html_file/$temp_filter</i></b></font></td></tr>\n";
        	print "  <tr><td align=right>Back to: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$filter_directory</i></b></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>2) Recreate & &nbsp</td><td align=left><font color=$filename_color><b><i>$tracker_directory/$html_file</i></b></font> directory</td></tr>\n";
        	print "  <tr><td align=right>Repopulate: &nbsp&nbsp</td><td align=left>&nbsp</td></tr>\n";
        } else {
        	print "  <tr><td align=right>1) Move: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$work_directory/$html_file/$temp_filter</i></b></font></td></tr>\n";
        	print "  <tr><td align=right>Back to: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$filter_directory</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>2) Move: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$work_directory/$html_file/$rrdtool_file</i></b></font></td></tr>\n";
        	print "  <tr><td align=right>Back to: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$rrdtool_directory</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>3) &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp</td><td>If this Tracking was on the Dashboard, either re-create the Dashnoard entry, or copy Dashboard file</i></b></font></td></tr>\n";
        	print "  <tr><td align=right>Back to: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$dashboard_directory</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>4) Recreate & &nbsp</td><td align=left><font color=$filename_color><b><i>$tracker_directory/$html_file</i></b></font> directory</td></tr>\n";
        	print "  <tr><td align=right>Repopulate: &nbsp&nbsp</td><td align=left>&nbsp</td></tr>\n";
        }
        print "  <tr><td>&nbsp</td></tr>\n";

        if ($#removed_groups > -1) {
        	print "  <tr><td colspan=2>This tracking component was removed from the following Groups:</td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
                foreach $group_label (@removed_groups) { print "  <tr><td>&nbsp</td><td align=left><font color=$filename_color><b><i>$group_label</i></b></font></td></tr>\n"; }
        	print "  <tr><td>&nbsp</td></tr>\n";
		print "  <tr><td colspan=2>If you recreate the Tracking, and you want to restore the listed groups back</td></tr>\n";
		print "  <tr><td colspan=2>to their original state, you will have to revise each Group and add the</td></tr>\n";
		print "  <tr><td colspan=2>Tracking back in.</td></tr>\n";
        }
        print "  </table>\n";

        print "  <table>\n";
       	print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td><form method=post action=\"$cgi_bin_short/FlowTracker_Management.cgi?List\">\n";
        print "  <button class=links type=submit>Return</button></form></td></tr>\n";
        print "  </table>\n";
        print "  </div>\n";
}

elsif ($action eq "Archive Tracking") {

        $filter_archive = $filter_file;
        $filter_archive =~ s/\.fil/\.archive/;
        $mv_command = "mv $filter_file $filter_archive";

        # Check to see if this archive is a group file

        $group_file = $filter_file;
        $group_file =~ s/\.fil/\.grp/;

        if (-e $group_file) {
                $is_group = 1;
                $filter_archive = $group_file;
                $filter_file    =~ s/\.fil/\.grp/;
                $filter_archive =~ s/\.grp/\.archive/;
                $mv_command = "mv $group_file  $filter_archive";
        }

        system($mv_command);

        if (!$is_group) {
                $rrdtool_archive = $rrdtool_file;
                $rrdtool_archive =~ s/\.rrd/\.archive/;
                $mv_command = "mv $rrdtool_file $rrdtool_archive";
                system($mv_command);
        }

	&create_UI_sides("FlowTracker");
	print " <div id=content_wide>\n";
	print "<br><b>Archive FlowTracking</b><br><br>\n\n";

        print "  <br>\n";
        print "  <table>\n";
        print "  <tr><td width=160 align=right>The Filter File: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$filter_file</i></b></font></td></tr>\n";
        print "  <tr><td>&nbsp</td><td>&nbsp</td></tr>\n";
        print "  <tr><td width=160 align=right>has been renamed: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$filter_archive</i></b></font></td></tr>\n";
        print "  <tr><td>&nbsp</td><td>&nbsp</td></tr>\n";
        if (!$is_group) {
        	print "  <tr><td align=right>The RRDtool File: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$rrdtool_file</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>has been renamed: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$rrdtool_archive</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td><td>&nbsp</td></tr>\n";
	}
        print "  <tr><td>&nbsp</td><td>&nbsp</td></tr>\n";
        print "  </table>\n";
        print "  <table>\n";
        print "<tr><td align=left>Links to the graphs have been moved to the Archived Trackings section at the bottom\n";
        print "of the Manage All FlowTrackings pulldown. FlowTracker_Collector will stop collecting\n";
        print "data for this Tracking. However, for historical purposes, the Tracking has not been\n";
        print "removed from any Groups it may belong to. You may restart this Tracking by clicking on\n";
        print "the Restart option now available for the listed tracking in the Achives section of the\n";
        print "the Manage All FlowTrackings pulldown. You may have to refresh your screen.</td></tr>\n";
        print "  </table>\n";

        print "  <table>\n";
       	print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td><form method=post action=\"$cgi_bin_short/FlowTracker_Management.cgi?List\">\n";
        print "  <button class=links type=submit>Return</button></form></td></tr>\n";
        print "  </table>\n";
        print "  </div>\n";
}

elsif ($action eq "Restart Tracking") {

        $filter_archive = $filter_file;
        $filter_archive =~ s/\.fil/\.archive/;

        # Check to see if this archive is not a group file

        $is_group = 0;
        open  (ARCHIVE_FILTER,"<$filter_archive");
        while (<ARCHIVE_FILTER>) {
                chop;
                $key = substr($_,0,8);
                if ($key eq " input: ") {
                        ($input,$field,$field_value) = split(/: /);
                        if ($field eq "tracking_type") {
				if ($field_value eq "Group") { 
					$is_group = 1;
					last;
				}
			}
		}
	}
        close (ARCHIVE_FILTER);

        if ($is_group) { $filter_file =~ s/\.fil/\.grp/; }

        # Move the archive back to it's original form

        $mv_command = "mv $filter_archive $filter_file";
        system($mv_command);

        if (!$is_group) {
                $rrdtool_archive = $rrdtool_file;
                $rrdtool_archive =~ s/\.rrd/\.archive/;
                $mv_command = "mv $rrdtool_archive $rrdtool_file";
                system($mv_command);
        }

	&create_UI_sides("FlowTracker");
	print " <div id=content_wide>\n";
	print "<br><b>Restart FlowTracking</b><br><br>\n\n";

        print "  <br>\n";
        print "  <table>\n";
        print "  <tr><td width=160 align=right>The archived Filter File: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$filter_archive</i></b></font></td></tr>\n";
        print "  <tr><td>&nbsp</td><td>&nbsp</td></tr>\n";
        print "  <tr><td width=160 align=right>has been renamed: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$filter_file</i></b></font></td></tr>\n";
        print "  <tr><td>&nbsp</td><td>&nbsp</td></tr>\n";
        if (!$is_group) {
        	print "  <tr><td align=right>The archived RRDtool File: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$rrdtool_archive</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td></tr>\n";
        	print "  <tr><td align=right>has been renamed: &nbsp&nbsp</td><td align=left><font color=$filename_color><b><i>$rrdtool_file</i></b></font></td></tr>\n";
        	print "  <tr><td>&nbsp</td><td>&nbsp</td></tr>\n";
	}
        print "  <tr><td>&nbsp</td><td>&nbsp</td></tr>\n";
        print "  </table>\n";
        print "  <table>\n";
        print "<tr><td align=left>Links to the graphs have been restored to the Active Trackings section at the top\n";
        print "of the Manage All FlowTrackings pulldown. You may have to refresh your screen. FlowTracker_Collector\n";
        print "will resume collecting data for this Tracking.</td></tr>\n";
        print "  </table>\n";

        print "  <table>\n";
       	print "  <tr><td>&nbsp</td></tr>\n";
        print "  <tr><td><form method=post action=\"$cgi_bin_short/FlowTracker_Management.cgi?List\">\n";
        print "  <button class=links type=submit>Return</button></form></td></tr>\n";
        print "  </table>\n";
        print "  </div>\n";

} else {
	&create_UI_sides("FlowTracker");
	print " <div id=content_wide>\n";
        print "  </div>\n";
}

&create_UI_service("FlowTracker","service_bottom",$filter_hash);
&finish_the_page("FlowTracker");
